# Roo Code 3.16 Release Notes (2025-05-12)

*These release notes include all the improvements from v3.16.x. Last updated with 3.16.6 on 2025-05-12*

This release introduces vertical tab navigation for settings, new API providers (Groq, Chutes, LiteLLM), clickable code references, Dutch localization, stability enhancements including tool loop detection and improved error handling, UI updates (including a temporary Tailwind CSS migration and its reversion), broader language support with Elixir, and improvements to provider profile management, alongside various bug fixes and miscellaneous updates.

## Gemini Model and Caching Updates
- The `gemini-2.5-pro-preview-05-06` model is now available for [Vertex](/providers/vertex) and [Google Gemini](/providers/gemini) providers. Users of the older `gemini-2.5-pro-preview-03-25` will automatically benefit from this newer model, as the previous ID now aliases to the latest version on Google's backend. No configuration changes are needed. (thanks zetaloop!)
- Prompt caching is now enabled by default for supported Gemini models on the [Vertex](/providers/vertex) and [Google Gemini](/providers/gemini) providers, leading to:
    - **Faster Responses for Repeated Queries**: Gemini remembers previous similar prompts.
    - **Reduced API Usage**: Minimizes redundant API calls.
    - **Simplified Experience with Opt-Out Control**: Active out-of-the-box, but can be disabled in settings.

## Total Settings Navigation Overhaul (thanks dlab-anton!)

FINALLY the settings interface has been revamped with a new vertical tab layout for a more efficient and intuitive experience
    - **One-Click Access:** Navigate between settings sections with a single click via the new vertical tabs in the settings view.
    - **Improved Layout and Clarity:** Settings are now organized in a clear vertical list for better visibility.
        
<img src="/img/v3.16/v3.16.gif" alt="Settings vertical tab navigation" width="800" />

## MCP Service Improvements

    - MCP server errors are now captured and shown in a new "Errors" tab (thanks robertheadley!)
    - Error logging will no longer break MCP functionality if the server is properly connected (thanks ksze!)

## Clickable Code References in Chat (thanks KJ7LNW!)

Navigating code discussed in AI responses is now significantly easier:
    - **Clickable Code and Filenames**: Any `code` or `filename.extension()` mentioned by the AI is now a clickable link.
    - **Jump to Specific Lines**: Links open the relevant file in your editor and navigate directly to the referenced line number.
    - **Streamlined Code Exploration**: Quickly move from AI explanations to the exact spot in your codebase.

<img src="/img/v3.16/v3.16.png" alt="Clickable code references in chat" width="450" />

## Tool Use Improvements
*   **Clarified XML Tool Formatting Instructions**: Documentation and prompts now provide clearer examples of how to format XML tool use, preventing the `<tool_name>` and other tool use errors. 
*   This fix is largely targeted at issues faced with Gemini 2.5 when using tools

## Continued UI/UX Improvements (thanks elianiva!)

General UI improvements for a more consistent, visually appealing, and intuitive experience:
    - **Visually Unified Design**: A more consistent look and feel across settings, prompt interactions, and mode selections.
    - **Improved Theme Adaptability**: Better consistency across different VS Code themes.
    - **Streamlined Interactions**: Tidied up UI elements like mode selection and prompt enhancement areas.
    - **Modernized Icons and Tooltips for Code Blocks**: Code block controls (copy, wrap, expand/collapse) now use crisp Lucide icons and feature translated tooltips for better accessibility.

<img src="/img/v3.16/v3.16-1.png" alt="General UI/UX improvements example" width="800" />

    *These are just a few examples of the many UI/UX improvements in this release.*

## New Provider: Groq Integration (thanks shariqriazz!)
You can now connect to [Groq](/providers/groq) and utilize their high-speed language models directly within the extension.

## New Provider: Chutes AI Integration (thanks shariqriazz!)
Support for [Chutes AI](/providers/chutes) has also been added, allowing you to leverage their specialized AI capabilities.

## QOL Improvements
### New Provider: LiteLLM Integration
We've introduced support for the [LiteLLM provider](/providers/litellm), simplifying access to a wide array of language models. This new integration offers:
*   **Automatic Model Discovery**: Roo Code now automatically fetches and lists available models from your LiteLLM server. This means users no longer need to manually configure each LiteLLM model within Roo Code, streamlining setup and making it easier to switch between models served by LiteLLM.
*   **Simplified Access to 100+ LLMs**: Leverage LiteLLM's ability to provide a unified OpenAI-compatible API for various underlying models.

<img src="/img/litellm/litellm.png" alt="Roo Code LiteLLM Provider Settings" width="600" />

This new provider significantly improves the ease of using diverse models through LiteLLM. For more details on setting up LiteLLM, see the [LiteLLM provider documentation](/providers/litellm).

### Tool Loop Detection
We've implemented a mechanism to detect and prevent tool execution loops, enhancing stability and user experience:
*   **Prevents Infinite Loops**: The system now identifies when a tool might be caught in a repetitive cycle and intelligently intervenes by prompting for user input.
*   **Improved Stability**: Reduces the risk of the application becoming unresponsive or stuck due to unintentional tool looping.

This ensures a smoother, more reliable, and frustration-free interaction with the extension's tools.
*   **Dutch Localization Added**: Added Dutch language support, allowing Dutch-speaking users to use the extension in their native language for a more inclusive experience. (thanks Githubguy132010!)
*   **Responsive Footer Buttons in About Section**: Fixed the layout of footer buttons in the About section, ensuring they wrap correctly on narrow screens for a better mobile experience and improved accessibility. (thanks ecmasx!)
*   Improve accessibility of auto-approve toggles (thanks Deon588!)
*   You can now toggle the `terminal.integrated.inheritEnv` VSCode setting directly for the Roo Code settings (thanks KJ7LNW!)
*   Improve provider profile management in the external API
*   Restore "Improve provider profile management in the external API"
*   Show properly formatted multi-line commands in preview (thanks KJ7LNW!)
*   Handle unsupported language errors gracefully in read_file tool (thanks KJ7LNW!)
*   Enhance focus styles in select-dropdown and fix docs URL (thanks zhangtony239!)
*   Properly handle mode name overflow in UI (thanks elianiva!)

## Provider Updates

*   Enforce provider selection in OpenRouter by using 'only' parameter and disabling fallbacks (thanks shariqriazz!)

## Bug Fixes
*   Fix migration and persistence of per-mode API profiles (thanks alasano!)
*   Fix usage of `path.basename` in the extension webview (thanks samhvw8!)
*   Fix display issue of the programming language dropdown in the code block component (thanks zhangtony239!)
*   Requesty provider fixes (thanks dtrugman!)
*   Fix display issues with long profile names (thanks cannuri!)
*   Prevent terminal focus theft on paste after command execution (thanks MuriloFP!)
*   Fix race condition when updating prompts (thanks elianiva!)
*   Fix display issues in high contrast themes (thanks zhangtony239!)
*   Fix not being able to use specific providers on Openrouter (thanks daniel-lxs!)
*   Fix project MCP always allow issue (thanks aheizi!)
*   Revert "Improve provider profile management in the external API" until we track down a bug with defaults
*   Fix to subtask sequencing (thanks wkordalski!)
*   Fix webview terminal output processing error (thanks KJ7LNW!)
*   Fix textarea empty string fallback logic (thanks elianiva!)

*   **Revert Tailwind Migration**: Restored the previous user interface by reverting the Tailwind CSS migration. This returns the UI to a familiar and stable state, resolving issues introduced by the migration and ensuring users see the interface as expected without unexpected layout or style changes.
*   **Add Elixir File Support in Language Parser**: Added support for Elixir (`.ex`, `.exs`) files in the language parser. This expands language support, allowing users to work with Elixir code seamlessly and enabling better code analysis for improved productivity. (thanks pfitz!)
*   **Improved Error Handling for Streaming**: Fixed an issue where the app could get stuck waiting for a response. The app now recovers gracefully from errors during streaming, reducing the likelihood of unresponsive behavior and improving reliability. (thanks monkeyDluffy6017!)
*   **Editor Name in Telemetry**: Added the editor name to telemetry data. This helps in understanding which editors are most used and enables more targeted improvements and support for different environments.
*   **Improved Evaluation Defaults and Setup**: Updated evaluation defaults and improved the setup process, making the evaluation environment easier and more reliable to configure with more practical out-of-the-box settings.
*   **Update Dependencies**: Updated dependencies to their latest versions for improved security and performance.
*   Ensure evals exercises are up-to-date before running evals (thanks shariqriazz!)
*   Organize provider settings into separate components
*   Add support for tests that use ESM libraries
*   Move environment detail generation to a separate module
*   Save OpenAI compatible custom headers correctly